@@ -62,11 +62,16 @@ sub parse_session_id
   my ($s) = @_;
   
   my $cookiename = $s->context->config->data_connections->session->cookie_name;
+
   no warnings 'uninitialized';
   if( my ($id) = $ENV{HTTP_COOKIE} =~ m/\b$cookiename\=([a-f0-9]+)\b/ )
   {
     return $id;
   }
+  elsif( ($id) = $s->context->r->headers_in->{Cookie} =~ m/\b$cookiename\=([a-f0-9]+)\b/ )
+  {
+    return $id;
+  }
   else
   {
     return;
@@ -150,7 +155,7 @@ sub retrieve
   my $now = time2iso();
   $sth->execute( $id );
   my ($data, $modified_on) = $sth->fetchrow;
-  $data = thaw($data);
+  $data = thaw($data) || { SessionID => $id };
   $sth->finish();
 
   if( time() - str2time($modified_on) >= ( $s->context->config->data_connections->session->session_timeout * 59 ) )
@@ -170,12 +175,17 @@ sub retrieve
   weaken($s);
   
   no warnings 'uninitialized';
-  $s->{__signature} = md5_hex(
+  
+  my @keys = sort keys(%$s);
+  
+  my $sig = md5_hex(
     join ":",
       map { "$_:$s->{$_}" } 
-        grep { $_ ne '__signature' } sort keys(%$s)
+        grep { $_ ne '__signature' } @keys
   );
   
+  $s->{__signature} = $sig;
+  
   return $s;
 }# end retrieve()
 
@@ -227,7 +237,8 @@ sub reset
 #==============================================================================
 sub new_session_id
 {
-  md5_hex( rand() );
+  my $s = shift;
+  md5_hex( $s->context->config->web->application_name . rand() );
 }# end new_session_id()
 
 
@@ -239,7 +250,7 @@ sub write_session_cookie
   my $state = $s->context->config->data_connections->session;
   my $cookiename = $state->cookie_name;
   $s->context->response->AddHeader(
-    'Set-Cookie' =>  "$cookiename=$s->{SessionID}; path=/; domain=" . $state->cookie_domain
+    'Set-Cookie' =>  "$cookiename=$s->{SessionID}; path=/;" #; domain=" . $state->cookie_domain
   );
   
   # If we weren't given an HTTP cookie value, set it here.
@@ -293,17 +304,56 @@ Within your ASP script:
 
 =head1 DESCRIPTION
 
-The global C<$Session> object is an instance of a subclass of C<Apache2::ASP::SessionStateManager>.
+The global C<$Session> object is an instance of a subclass of C<Apache2::ASP::SessionStateManager>
+or one of its subclasses.
 
 It is a blessed hash that is persisted within a database.  Use it to share information across all requests for
 all users.
 
-B<NOTE:> - do not store database connections within the C<$Session> object because they cannot be shared across
+B<NOTE:> - do not store database connections or filehandles within the C<$Session> object because they cannot be shared across
 different processes/threads at this time.
 
 =head1 METHODS
 
-TBD
+=head2 save( )
+
+Stores the Session object in the database.  Returns true.
+
+=head1 CONFIGURATION
+
+=head2 XML Config
+
+The file C<apache2-asp-config.xml> should contain a section like the following:
+
+  <?xml version="1.0"?>
+  <config>
+    ...
+    <data_connections>
+      ...
+      <session>
+        <manager>Apache2::ASP::SessionStateManager::MySQL</manager>
+        <cookie_name>session-id</cookie_name>
+        <dsn>DBI:mysql:dbname:localhost</dsn>
+        <username>sa</username>
+        <password>s3cr3t!</password>
+        <session_timeout>30</session_timeout>
+      </session>
+      ...
+    </data_connections>
+    ...
+  </config>
+
+=head2 Database Storage
+
+The database named in the XML config file should contain a table like the following:
+
+  CREATE TABLE  asp_sessions (
+    session_id    char(32) NOT NULL,
+    session_data  blob,
+    created_on    datetime default NULL,
+    modified_on   datetime default NULL,
+    PRIMARY KEY  (session_id)
+  ) ENGINE=InnoDB DEFAULT CHARSET=latin1
 
 =head1 BUGS
 
@@ -318,7 +368,7 @@ of Apache2::ASP in action.
 
 =head1 AUTHOR
 
-John Drago L<mailto:jdrago_999@yahoo.com>
+John Drago <jdrago_999@yahoo.com>
 
 =head1 COPYRIGHT AND LICENSE
 
